extern crate libc;

use std::num::NonZeroUsize;
use std::io::Error;
use std::io::ErrorKind;
use std::io::Result;

fn available_parallelism1() -> Result<NonZeroUsize> {
    unsafe {
        let set = libc::_cpuset_create();
        if !set.is_null() {
            let mut count: usize = 0;
            if libc::pthread_getaffinity_np(libc::pthread_self(),
                    libc::_cpuset_size(set), set) == 0 {
                for i in 0..u64::MAX {
                    match libc::_cpuset_isset(i, set) {
                        -1 => break,
                        0 => continue,
                        _ => count = count + 1,
                    }
                }
            }
            libc::_cpuset_destroy(set);
            if let Some(count) = NonZeroUsize::new(count) {
                return Ok(count);
            }
        }
    }
    Err(Error::new(ErrorKind::Other, "bad"))
}

fn available_parallelism2() -> Result<NonZeroUsize> {
    let set = unsafe { libc::_cpuset_create() };
    if set.is_null() {
        return Err(Error::new(ErrorKind::Other, "null"))
    } else {
        let mut count: usize = 0;
        if unsafe { libc::pthread_getaffinity_np(libc::pthread_self(),
                libc::_cpuset_size(set), set) } == 0 {
            for i in 0..u64::MAX {
                match unsafe { libc::_cpuset_isset(i, set) } {
                    -1 => break,
                    0 => continue,
                    _ => count = count + 1,
                }
            }
        }
        unsafe { libc::_cpuset_destroy(set) };
        if let Some(count) = NonZeroUsize::new(count) {
            return Ok(count);
        } else {
            println!("count={}", count);
            return Err(Error::new(ErrorKind::Other, "zero"))
        }
    }
    Err(Error::new(ErrorKind::Other, "bad"))
}

pub fn main() {
    match available_parallelism1() {
        Ok(n) => println!("ok {}", n),
        Err(e) => println!("err {}", e),
    }
}
